/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.genome;

import java.io.PrintStream;
import java.util.List;
/**
 * Interface for classes that offer services of storage and retrieval of information about
 * reference sequences as assemblies, contigs or transcriptomes
 * @author Jorge Duitama
 */
public interface Genome {
	/**
	 * @return List<String> Sequence names making the genome
	 */
	public List<String> getSequenceNames();
	/**
	 * Retrieves the reference base at the given genomic coordinate
	 * @param sequenceName Name of the sequence to look for
	 * @param absolutePosition One based position relative to the start of the sequence with
	 * the given name 
	 * @return char Reference base
	 */
	public char getReferenceBase(String sequenceName, int absolutePosition);
	/**
	 * Changes the reference base at the given genomic coordinate
	 * @param sequenceName Name of the sequence to look for
	 * @param absolutePosition One based position relative to the start of the sequence with
	 * the given name  
	 * @param base New base to set.
	 */
	public void setReferenceBase(String sequenceName, int absolutePosition, char base);
	/**
	 * Prints the genome in the given stream
	 * @param out Stream to print the genome in fasta format
	 * @param lineLength Number of bases per line
	 */
	public void saveGenome(PrintStream out, int lineLength);
}
