/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.genome;
/**
 * Interface for defined regions in a reference genome. Regions can be many types of variants like
 * SNPs, Indels or CNVs or they also can be Exons, Transcripts, Genes or Contigs.
 * This common interface helps to implement common tasks like sorting or overlaps finding 
 * @author Jorge Duitama
 *
 */
public interface GenomicRegion {
	/**
	 * @return String name of the sequence where the region is mapped
	 */
	public String getSequenceName();
	/**
	 * @return First position of the region relative to the sequence
	 */
	public int getStart();
	/**
	 * @return Last position of the region relative to the sequence
	 */
	public int getEnd(); //Last position of the region always greater than start
	/**
	 * @return int length of the region. Typically equal to end - start + 1
	 */
	public int length();
}
