/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.genome;

import java.util.Comparator;
import java.util.List;
/**
 * Comparator for two genomic regions. Compares sequence names according with a given sorted list
 * of possible sequence names. For equal names, uses GenomicRegionPositionComparator
 * @author Jorge Duitama
 */
public class GenomicRegionComparator implements Comparator<GenomicRegion> {
	private List<String> sequenceNames;
	public GenomicRegionComparator(List<String> sequenceNames) {
		this.sequenceNames = sequenceNames;
	}
	@Override
	public int compare(GenomicRegion r1, GenomicRegion r2) {
		if(!r1.getSequenceName().equalsIgnoreCase(r2.getSequenceName())) {
			return findPosition(r1.getSequenceName())-findPosition(r2.getSequenceName());
		}
		return GenomicRegionPositionComparator.getInstance().compare(r1, r2);
	}
	private int findPosition(String sequenceName) {
		for(int i=0;i<sequenceNames.size();i++) {
			if(sequenceNames.get(i).equalsIgnoreCase(sequenceName)) {
				return i;
			}
		}
		return sequenceNames.size();
	}
}
