/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.main;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

import edu.uconn.ngstools.genome.GenomicRegionComparator;
import edu.uconn.ngstools.readalignments.ReadAlignment;
import edu.uconn.ngstools.readalignments.filehandlers.SAMReadAlignmentsHandler;

/**
 * Program that receives a list of SAM files assumed to be grouped by sequence name 
 * and with alignments to the same sequence sorted by start position and merges them 
 * according with this sorting mechanism into a single file
 * @author Jorge Duitama
 */
public class SAMFilesMerger {

	public static void main(String [] args) throws Exception {
		SAMReadAlignmentsHandler handler = new SAMReadAlignmentsHandler();
		handler.loadSequenceNames(args[0]);
		PrintStream out = System.out;
		out.println("@HD\tVN:1.0\tSO:coordinate");
		SAMReadAlignmentsHandler.copySequenceHeaders(args[0], out);
		out.println("@PG\tID:"+SAMReadAlignmentsHandler.class.getName());
		GenomicRegionComparator cmp = new GenomicRegionComparator(handler.getSequenceNames());
		BufferedReader [] streams = new BufferedReader [args.length];
		ReadAlignment [] alns = new ReadAlignment[args.length];
		//Open buffers
		for(int i=0;i<args.length;i++) {
			FileInputStream fis = new FileInputStream(args[i]);
			BufferedReader in = new BufferedReader(new InputStreamReader(fis));
			streams[i] = in;
			alns[i] = handler.createReadAlignment(in);
		}
		//Merge files
		while(true) {
			int nextToPrint = -1;
			for(int i=0;i<alns.length;i++) {
				if(alns[i]!=null) {
					if(nextToPrint==-1 || cmp.compare(alns[i], alns[nextToPrint]) < 0) {
						nextToPrint = i;
					}
				}
			}
			if(nextToPrint >=0) {
				handler.saveReadAlignment(alns[nextToPrint], out);
				alns[nextToPrint] = handler.createReadAlignment(streams[nextToPrint]);
			} else {
				break;
			}
		}
		//Close streams
		for(int i=0;i<streams.length;i++) {
			streams[i].close();
		}
	}

}
