/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.main;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.List;

import edu.uconn.ngstools.genome.Genome;
import edu.uconn.ngstools.genome.GenomeAssembly;
import edu.uconn.ngstools.transcriptome.CCDSTranscriptome;
import edu.uconn.ngstools.variants.SNV;
import edu.uconn.ngstools.variants.filehandlers.SNVSFileHandler;
import edu.uconn.ngstools.variants.filehandlers.SimpleSNVSFileHandler;

/**
 * Program that mofdifies a public reference genome according with a given set of variants 
 * specific to the type of organism being studied
 * @author Jorge Duitama
 */
public class SNVSReferenceGenerator {
	private String mapFilename;
	private String referenceFilename;
	private String snpsFilename;
	private String outFilename;
	private int lineLength=100;
	public void generateStrainReference() throws IOException {
		Genome genome = loadGenome();
		SNVSFileHandler snvsHandler = new SimpleSNVSFileHandler();
		List<SNV> snvs = snvsHandler.loadSNVS(snpsFilename);
		for(SNV snv: snvs) {
			genome.setReferenceBase(snv.getSequenceName(), snv.getPosition(), snv.getAltBase());
		}
		PrintStream out = new PrintStream(new FileOutputStream(outFilename));
		genome.saveGenome(out, lineLength);
		out.flush();
		out.close();
	}
	private Genome loadGenome() throws IOException {
		if(mapFilename!=null) {
			return new CCDSTranscriptome(mapFilename, referenceFilename);
		}
		return new GenomeAssembly(referenceFilename);
	}
	public static void main(String[] args) throws Exception {
		SNVSReferenceGenerator generator = new SNVSReferenceGenerator();
		int i=0;
		while(i<args.length && args[i].charAt(0)=='-') {
			if("-mapFile".equalsIgnoreCase(args[i])) {
				i++;
				generator.mapFilename = args[1];
			} else if("-l".equalsIgnoreCase(args[i])) {
				i++;
				generator.lineLength = Integer.parseInt(args[i]);
			}
			i++;
		}
		generator.referenceFilename = args[i];
		i++;
		generator.snpsFilename = args[i];
		i++;
		generator.outFilename = args[i];
		i++;
		generator.generateStrainReference();
	}
}
