/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.readalignments;

import java.util.Comparator;
/**
 * Comparator for read alignments which makes two alignments equal if they span a common region
 * @author Jorge Duitama
 *
 */
public class CommonSpanningAlignmentComparator implements Comparator<ReadAlignment> {

	@Override
	public int compare(ReadAlignment aln1, ReadAlignment aln2) {
		
		if(!aln1.getSequenceName().equals(aln2.getSequenceName())) {
			return aln1.getSequenceName().compareTo(aln2.getSequenceName());
		}
		if (aln1.getStart() < aln2.getEnd() && aln2.getStart() < aln1.getEnd()) {
			return 0;
		}
		return aln1.getStart()-aln2.getStart();
	}

}
