/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.sequences;
/**
 * Implementation of AbstractSequence for DNA Sequences whithout degenerate bases
 * @author Jorge
 */
public class DNASequence extends AbstractSequence{
	private static String bases = "ACGT";
	/**
	 * Creates a new DNA sequence with the given bases
	 * @param sequence String with the bases for the new sequence
	 */
	public DNASequence(String sequence) {
		this.setSequence(sequence);
	}
	/**
	 * Creates a new empty DNA Sequence
	 */
	public DNASequence() {
	}
	@Override
	public String getAlphabet() {
		return bases;
	}
	@Override
	protected int getBitsPerCharacter() {
		return 2;
	}
	public static boolean isInAlphabeth(char base) {
		return bases.indexOf(base)>=0;
	}
	//TODO: Improve this method by avoiding transferring the sequence to a string
	/**
	 * @return String Reverse complement of this sequence as a String object
	 */
	public String getReverseComplement() {
		StringBuffer sb = new StringBuffer();
		String seq = toString();
		for(int i=seq.length()-1;i>=0;i--) {
			sb.append(getComplement(seq.charAt(i)));
		}
		return sb.toString();
	}
	/**
	 * Gets the complement of the given base
	 * @param base to complement
	 * @return char Complement of the given base
	 */
	public static char getComplement(char base) {
		int index = bases.indexOf(base);
		if(index>=0) {
			return bases.charAt(bases.length()-index-1);
		}
		return base;
	}
}
