/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.sequences;
/**
 * Implementation of AbstractSequence for DNA Sequences with degenerate bases
 * @author Jorge Duitama
 *
 */
public class DegenerateSequence extends AbstractSequence {
	public static final String bases = "ACGT";
	public static final String [] iubCodes2 = {"AMRW","MCSY","RSGK","WYKT"};
	public static final String iubCodes3 = "BDHV";
	public static final String alphabeth = "ACGTMRWSYKBDHVN";
	
	@Override
	public String getAlphabet() {
		return alphabeth;
	}
	/**
	 * Given a degenerate base calculate the bases that it represents
	 * @param base Degenerate base
	 * @return String The same base if it is not degenerate. The two or three bases concatenated
	 * if it is a degenerate base or null if it is not found
	 */
	public static String getExtendedBases(char base) {
		if(base == 'N') return bases;
		int index = bases.indexOf(base); 
		if(index>=0) {
			return ""+base;
		}
		index = iubCodes3.indexOf(base);
		if(index>=0) {
			StringBuffer answer = new StringBuffer();
			for(int i=0;i<bases.length();i++) {
				if(i!=index) {
					answer.append(bases.charAt(i));
				}
			}
			return answer.toString();
		}
		for(int i=0;i<iubCodes2.length;i++) {
			index = iubCodes2[i].indexOf(base);
			if(index>=0) {
				return""+bases.charAt(i)+""+bases.charAt(index);
			}
		}
		return null;
	}
	/**
	 * Returns the degenerate base corresponding with the two non degenerate given bases
	 * @param base1 First non degenerate base
	 * @param base2 Second non degenerate base
	 * @return char Degenerate base corresponding with the given two bases
	 */
	public static char getDegenerateBase(char base1, char base2) {
		int i = bases.indexOf(base1);
		int j = bases.indexOf(base2);
		return iubCodes2[i].charAt(j);
	}
}
