/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.transcriptome;

import edu.uconn.ngstools.genome.GenomicRegion;

/**
 * Information for an Exon
 * @author Jorge Duitama
 */
public class Exon implements GenomicRegion {
	private Gene gene;
	private int start;
	private int end; //Always greater or equal than start
	/**
	 * Creates a new Exon with the given information
	 * @param gene Gene where the exon is located
	 * @param start First position of the exon in the sequence where the gene is located
	 * @param end Last position of the exon in the sequence where the gene is located
	 */
	public Exon(Gene gene, int start, int end) {
		this.gene = gene;
		this.setStart(start);
		this.setEnd(end);
	}
	/**
	 * @return int First position of the exon in the sequence where the gene is located
	 */
	public int getStart() {
		return start;
	}
	/**
	 * Changes the start of the exon
	 * @param start New Start
	 */
	public void setStart(int start) {
		this.start = start;
	}
	/**
	 * @return int Last position of the exon in the sequence where the gene is located
	 */
	public int getEnd() {
		return end;
	}
	/**
	 * Changes the end of the exon
	 * @param end New end
	 */
	public void setEnd(int end) {
		this.end = end;
	}
	/**
	 * @return int length of the exon calculated as end-start+1 
	 */
	public int length() {
		return end-start+1;
	}
	/**
	 * @return String Sequence name where the gene is located
	 */
	public String getSequenceName() {
		return gene.getSequenceName();
	}
	/**
	 * @return Gene Object with the information of the gene where this exon belongs
	 */
	public Gene getGene() {
		return gene;
	}
	
}
