/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.transcriptome;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
/**
 * Information for a Gene mapped to a reference genome
 * @author Jorge Duitama
 */
public class Gene {
	private String sequenceName;
	private String id;
	private String name;
	private List<Transcript> transcripts = new ArrayList<Transcript>();
	/**
	 * Creates a new Gene with the given information
	 * @param sequenceName Name of the sequence where the Gene is mapped
	 * @param id Id of the gene
	 * @param name Name of the gene
	 */
	public Gene(String sequenceName, String id, String name) {
		super();
		this.sequenceName = sequenceName;
		this.id = id;
		this.name = name;
	}
	/**
	 * @return String Name of the sequence where this gene is mapped
	 */
	public String getSequenceName() {
		return sequenceName;
	}
	/**
	 * Changes the name of the sequence where this gene is mapped
	 * @param sequenceName New Sequence name
	 */
	public void setSequenceName(String sequenceName) {
		this.sequenceName = sequenceName;
	}
	/**
	 * Adds the given transcript to this gene
	 * @param t New transcript
	 */
	public void addTranscript(Transcript t) {
		transcripts.add(t);
	}
	/**
	 * @return List of transcripts added to this gene
	 */
	public Collection<Transcript> getTranscripts() {
		return transcripts;
	}
	/**
	 * @return String id of the gene
	 */
	public String getId() {
		return id;
	}
	/**
	 * Changes the id of the gene
	 * @param id New id
	 */
	public void setId(String id) {
		this.id = id;
	}
	/**
	 * @return String Name of the gene
	 */
	public String getName() {
		return name;
	}
	/**
	 * Changes the name of the gene
	 * @param name New gene name
	 */
	public void setName(String name) {
		this.name = name;
	}
}
