/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.variants;

import edu.uconn.ngstools.genome.GenomicRegion;

/**
 * Information for a single nucleotide polymorfism (SNV).
 * Implements the GenomicRegion interface by having assigning position to both the
 * start and the end
 * @author Jorge Duitama
 */
public class SNV implements GenomicRegion { 
	private String sequenceName;
	private int position;
	private char refBase;
	private char altBase;
	/**
	 * Creates a new SNV with the given information
	 * @param sequenceName Name of the sequence where the SNV is located
	 * @param position Position in the sequence where the SNV is located
	 * @param refBase Base present in the reference sequence in the given position
	 * @param altBase Alternative base for the given position 
	 */
	public SNV (String sequenceName, int position, char refBase, char altBase) {
		this.setSequenceName(sequenceName);
		this.setPosition(position);
		this.setRefBase(refBase);
		this.setAltBase(altBase);
	}
	/**
	 * @return String Name of the sequence where the SNV is located
	 */
	public String getSequenceName() {
		return sequenceName;
	}
	/**
	 * Changes the name of the sequence where the SNV is located
	 * @param sequenceName New name of the sequence
	 */
	public void setSequenceName(String sequenceName) {
		this.sequenceName = sequenceName;
	}
	/**
	 * @return int Position in the sequence where the SNV is located
	 */
	public int getPosition() {
		return position;
	}
	/**
	 * Changes the position in the sequence where the SNV is located
	 * @param position New coordinate
	 */
	public void setPosition(int position) {
		this.position = position;
	}
	/**
	 * @return char Reference base
	 */
	public char getRefBase() {
		return refBase;
	}
	/**
	 * Changes the reference base
	 * @param refBase New reference base
	 */
	public void setRefBase(char refBase) {
		this.refBase = refBase;
	}
	/**
	 * @return char Alternative base
	 */
	public char getAltBase() {
		return altBase;
	}
	/**
	 * Changes the alternative base
	 * @param altBase New alternative base
	 */
	public void setAltBase(char altBase) {
		this.altBase = altBase;
	}
	@Override
	public int getStart() {
		return position;
	}
	@Override
	public int getEnd() {
		return position;
	}
	@Override
	public int length() {
		return 1;
	}
}
