/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.variants.filehandlers;
/**
 * Helper class to transfer probabilities in phred scale forth and back
 * @author Jorge Duitama
 */
public class PhredScoreHelper {
	/**
	 * Calculates the phred score given the probability
	 * @param p Probability to calculate
	 * @return short score=-10*Math.log10(p). 255 if the probability is zero or the score is higher than 255
	 */
	public static short calculatePhredScore(double p) {
		if(p==0) {
			return 255;
		}
		short score = (short)Math.round(-10*Math.log10(p));
		if(score > 255) {
			return 255;
		}
		return score;
	}
	/**
	 * Calculates the probability related with the given score
	 * @param phredScore Score in Phred scale
	 * @return double probability=Math.pow(10.0, (-phredScore/10.0)). Zero if the score is equal to 255
	 */
	public static double calculateProbability(short phredScore) {
		if(phredScore == 255) {
			return 0; 
		}
		return Math.pow(10.0, (-phredScore/10.0));
	}
}
