/*******************************************************************************
 * NGSTools - Java tools for analysis of Next Generation Sequencing (NGS) data
 * Copyright 2010 Jorge Duitama
 *
 * This file is part of NGSTools.
 *
 *     NGSTools is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     NGSTools is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with NGSTools.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package edu.uconn.ngstools.variants.filehandlers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import edu.uconn.ngstools.variants.SNV;
/**
 * Loader that handles files with at least the following four columns separated 
 * either by space or tab:
 * <ul>
 * <li>Sequence name
 * <li>Position
 * <li>Reference base
 * <li>Alternative base
 * </ul>
 * 
 * @author Jorge Duitama
 */
public class SimpleSNVSFileHandler implements SNVSFileHandler {
	private String separator = " |\t";
	@Override
	public List<SNV> loadSNVS(String snpsFilename) throws IOException {
		List<SNV> answer = new ArrayList<SNV>();
		FileInputStream fis = new FileInputStream(snpsFilename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split(separator);
			SNV snv = new SNV(items[0],Integer.parseInt(items[1]),items[2].charAt(0),items[3].charAt(0));
			answer.add(snv);
			line=in.readLine();
		}
		fis.close();
		return answer;
	}

}
