package edu.uconn.bioinformatics.epitopeFinder;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.TreeMap;

public class NetMHCProteinEpitopeChecker implements ProteinEpitopeChecker {
	private String motifs;
	private String programName;
	private String tempDir="";
	public static final int BUFFER_MAX_SIZE = 300;
	private static Random random = new Random();
	private static boolean debug = false;
        private String algorithm;
	
	public NetMHCProteinEpitopeChecker () {
		ResourceBundle propsBundle = ResourceBundle.getBundle("epitopeFinder");
		programName = propsBundle.getString("netmhc.program");
		if(propsBundle.containsKey("netmhc.tempdir")) {
			String t = propsBundle.getString("netmhc.tempdir");
			if(t!=null) {
				tempDir = t+System.getProperty("path.separator");
			}
		}
	}
	@Override
	public List<EpitopeHit> findEpitopes(String proteinSequence) throws Exception {
		return findEpitopes(proteinSequence,0);
	}

	@Override
	public List<EpitopeHit> findEpitopes(String proteinSequence, double minScore) throws Exception {
		List<EpitopeHit> answer = new ArrayList<EpitopeHit>();
		File tempFile = makeFile(proteinSequence);
		for(int l=8;l<=11;l++) {
//			if (debug) {
//	              System.out.println("in findEpitopes(String proteinSequence, double minScore)"+proteinSequence);
//			}
			List<EpitopeHit> hits = findEpitopes(tempFile,minScore,l).get(proteinSequence);
			if(hits!=null) {
				answer.addAll(hits);
			}
			
		}
		tempFile.delete();
		return answer;
	}

	@Override
	public double getScore(String epitope, String motif) throws Exception {
		File tempFile = makeFile(epitope);
		List<EpitopeHit> hits = findEpitopes(tempFile,Integer.MIN_VALUE,epitope.length()).get(epitope);
		double score = -100;
		if(hits!=null) {
			for(EpitopeHit hit:hits) {
				if(hit.getMotif().equals(motif)) {
					score = hits.get(0).getScore();
				}
			}
		}
		tempFile.delete();
//sahar debug                
//                            System.err.println("in getScore motif, epitopescore");
//                            System.err.println(motif+" "+epitope+" "+score);
		return score;
	}

	@Override
	public void setMotifs(String motifs) {
		this.motifs = motifs;
		
	}

	@Override
        public void setAlgorithm(String algorithm){
            this.algorithm = algorithm;
        }

        @Override
	public Map<String, List<EpitopeHit>> findEpitopes(Set<String> proteinSequences, double minScore) throws Exception {
		Map<String,List<EpitopeHit>> answer = new TreeMap<String, List<EpitopeHit>>();
		List<String> buffer = new ArrayList<String>();
		for(String protein:proteinSequences) {
			if(buffer.size()==BUFFER_MAX_SIZE) {
				answer.putAll(findEpitopes(buffer, minScore));
				buffer.clear();
			}
			buffer.add(protein);
		}
		if(buffer.size()>0) {
			answer.putAll(findEpitopes(buffer, minScore));
		}
		return answer;
	}
	private Map<String,List<EpitopeHit>> findEpitopes(List<String> buffer, double minScore) throws IOException {
		Map<String,List<EpitopeHit>> answer = new TreeMap<String, List<EpitopeHit>>();
		File tempFile = makeFile(buffer);
//                System.out.println("in find epitopes");
		for(int l=8;l<=11;l++) {
			Map<String,List<EpitopeHit>> hitsMap = findEpitopes(tempFile, minScore, l);
			for(String protein:hitsMap.keySet()) {
				List<EpitopeHit> hits = answer.get(protein);
				if(hits==null) {
					hits = new ArrayList<EpitopeHit>();
					answer.put(protein, hits);
//                                        System.out.println(protein+" No results");
				}
//                                else
                                //sahar debugging missing Apc epitope
//                                if (protein.equals("LLEQIRAYCETCWElQEAHEQGMDQDKNP") ||
//                                        protein.equals("LLEQIRAYCETCWELQEAHEQGMDQDKNP") ||
//                                       protein.equals("LLEQIRAYCETCWEWQEAHEQGMDQDKNP"))// && l == 9)
//                                    for (EpitopeHit hit:hits) {
//                                        if (hit.getMotif().equals("H2-Kd")) {
//                                            System.out.println(protein+" "+hit.getEpitope()+" "+hit.getScore());
//                                        }
//                                    }

				hits.addAll(hitsMap.get(protein));
			}
		}
		tempFile.delete();
		return answer;
	}
	private Map<String,List<EpitopeHit>> findEpitopes(File pFile, double minScore, int length) throws IOException {
		Map<String,List<EpitopeHit>> answer = new TreeMap<String,List<EpitopeHit>>();
		String command = programName + " -a "+motifs+" -l "+length+" -s "+pFile.getPath();
//		System.out.println(command);
		Process p = Runtime.getRuntime().exec(command);  
        BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
        String line = in.readLine();
        while(line!=null) {
        	//System.out.println(line);
        	line = line.trim();
        	String [] items = line.split("\\s+");
        	if(items.length>4 && motifs.contains(items[items.length-1])) {
        		//System.out.println("Items: "+items.length);
        		//for(int i=0;i<items.length;i++) {
        		//	System.out.println("Item "+i+": "+items[i]);
        		//}
        		String motif = items[items.length-1];
        		String epitope = items[1];
        		int position = Integer.parseInt(items[0]);
                        double score;
                        if (algorithm.equals("ANN"))
                            score = getScore(items[3]);
                        else
                             score = getScore(items[2]);
        		String protein = items[items.length-2];
        		if(score >=minScore) {
        			List<EpitopeHit> hits = answer.get(protein);
        			if(hits == null) {
        				hits = new ArrayList<EpitopeHit>();
        				answer.put(protein, hits);
        			}
        			hits.add(new EpitopeHit(motif, epitope, position, score));
        		}
//sahar debug
                        else {
//                            System.err.println("in FindEpitopes score < minScore (motif, epitope, position, score)");
//                            System.err.println(motif+" "+epitope+" "+position+" "+score);
                        }
        	}
        	line = in.readLine();
        }
        BufferedReader inE = new BufferedReader(new InputStreamReader(p.getErrorStream()));
		line = inE.readLine();
		while(line!=null) {
//			System.err.println(line);
			line = inE.readLine();
		}
		return answer;
	}
	private double getScore(String scoreStr) {
		return Double.parseDouble(scoreStr);
	}
	private File makeFile(String proteinSequence) throws IOException {
		File f = new File(tempDir+Math.abs(random.nextInt())+"_"+proteinSequence+".tmp");
		FileOutputStream fos = new FileOutputStream(f);
		PrintWriter out = new PrintWriter(fos);
		out.println(">"+proteinSequence);
		out.println(proteinSequence);
		out.flush();
		out.close();
		return f;
	}
	private File makeFile(List<String> proteinSequences) throws IOException {
		File f = new File(tempDir+Math.abs(random.nextInt())+"_"+"all_proteins.tmp");
		FileOutputStream fos = new FileOutputStream(f);
		PrintWriter out = new PrintWriter(fos);
		for(String proteinSequence:proteinSequences) {
			out.println(">"+proteinSequence);
			out.println(proteinSequence);
		}
		out.flush();
		out.close();
		return f;
	}

}
