package edu.uconn.bioinformatics.epitopeFinder;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;


import edu.uconn.bioinformatics.mapping.RepeatFlagsCalculator;
import edu.uconn.ngstools.genome.GenomeAssembly;

public class Primer3Helper {
	public static final String COMMAND_CREATE_INPUT = "create-input";
	public static final String COMMAND_MERGE_OUTPUT = "merge-output";
	
	static String separator = "\t";
	private void generatePrimer3Input(String referenceFilename, String reportFilename, PrintStream out) throws IOException {
		GenomeAssembly genome = new GenomeAssembly(referenceFilename);
		Set<String> snpsIncluded = new TreeSet<String>();
		FileInputStream fis = new FileInputStream(reportFilename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split(separator);
			String seqName = items[0];
			String snp = items[1];
			String seqIdPCR = getSequenceId(items, "PCR");
			String seqIdSequencing = getSequenceId(items, "Sequencing");
			if(snpsIncluded.contains(seqIdPCR)) {
				line=in.readLine();
				continue;
			}
			snpsIncluded.add(seqIdPCR);
			//Sequence for PCR primers design
			String sequence = RepeatFlagsCalculator.getSequenceForPrimersDesign(genome, seqName, Integer.parseInt(snp), RepeatFlagsCalculator.OFFSET);
			int productLength = 1;
			//TODO: Calculate numbers as a function of the offset
			String excRegPCR = "151,"+(productLength+500);
			String sizeRangePCR = ""+(productLength+500)+"-"+(productLength+800);
			String excRegSeq = "1,150 301,"+(productLength+200)+" "+(productLength+650)+",150";
			String sizeRangeSeq = ""+(productLength+200)+"-"+(productLength+500);
			printPrimer3Record(out,seqIdPCR, sequence, excRegPCR, sizeRangePCR);
			printPrimer3Record(out,seqIdSequencing, sequence, excRegSeq, sizeRangeSeq);
			out.flush();
			line=in.readLine();
		}
		out.flush();
		in.close();
	}
	public String getSequenceId (String [] items, String suffix) {
		return items[0]+"|"+items[1]+"|"+suffix;
	}
	
	private void printPrimer3Record(PrintStream out, String seqId, String seq, String excludedRegions, String sizeRange) {
		out.println("PRIMER_SEQUENCE_ID="+seqId);
		out.println("SEQUENCE="+seq);
		out.println("EXCLUDED_REGION="+excludedRegions);
		out.println("PRIMER_MIN_SIZE=20");
		out.println("PRIMER_OPT_SIZE=25");
		out.println("PRIMER_MAX_SIZE=30");
		out.println("PRIMER_MIN_TM=55");
		out.println("PRIMER_OPT_TM=60");
		out.println("PRIMER_MAX_TM=65");
		out.println("PRIMER_PRODUCT_SIZE_RANGE="+sizeRange);
		out.println("PRIMER_TM_SANTALUCIA=1");
		out.println("PRIMER_SALT_CORRECTIONS=1");
		out.println("PRIMER_LOWERCASE_MASKING=1");
		out.println("=");
	}
	private void mergeOutput(String reportFilename, String primer3OutputFilename, PrintStream out) throws IOException{
		Map<String,String> snpsFound = new TreeMap<String,String>();
		FileInputStream fis = new FileInputStream(reportFilename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		FileInputStream fisPrimer3 = new FileInputStream(primer3OutputFilename);
		BufferedReader inPrimer3 = new BufferedReader(new InputStreamReader(fisPrimer3));
		String lastPrimer3OutputLine="";
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split(separator);
			out.print(line+separator);
			String seqIdPCR = getSequenceId(items, "PCR");
			String seqIdSequencing = getSequenceId(items, "Sequencing");
			String primersInfo = snpsFound.get(seqIdPCR);
			if(primersInfo==null) {
				StringBuffer primersInfoBuff = new StringBuffer();
				lastPrimer3OutputLine = getPrimersInfo(seqIdPCR,inPrimer3,lastPrimer3OutputLine,primersInfoBuff,false);
				primersInfoBuff.append(separator);
				lastPrimer3OutputLine = getPrimersInfo(seqIdSequencing,inPrimer3,lastPrimer3OutputLine,primersInfoBuff,true);
				primersInfo = primersInfoBuff.toString();
				snpsFound.put(seqIdPCR,primersInfo);
			}
			out.println(primersInfo);
			out.flush();
			line=in.readLine();
		}
		out.flush();
		inPrimer3.close();
		in.close();
	}


	private String getPrimersInfo(String seqId, BufferedReader in, String lastLine, StringBuffer out, boolean printFlags) throws IOException {
		String lineId = "PRIMER_SEQUENCE_ID="+seqId;
		String emptyLine = "-"+separator+"-"+separator+"-"+separator+"-"+separator+"-"+separator+"-"+separator+"-";
		StringBuffer seqInfo = new StringBuffer();
		if(printFlags) {
			seqInfo.append(""+separator+"-"+separator+"-"+separator+"-");
		}
		String line=lastLine;
		while (line!=null && !line.equals(lineId)) {
			line=in.readLine();
		}
		if (line==null) {
			out.append(emptyLine+seqInfo);
			return line;
		}
		String leftPrimer = null;
		String rightPrimer =null;
		String leftTm = null;
		String rightTm = null;
		String productSize = null;
		String sequence = null;
		line=in.readLine();
		while (line!=null && !line.startsWith("PRIMER_SEQUENCE_ID")) {
			if(line.startsWith("PRIMER_ERROR")) {
				out.append(emptyLine+seqInfo);
				return line;
			} else if (line.startsWith("PRIMER_LEFT_SEQUENCE")) {
				leftPrimer = line.substring(line.indexOf('=')+1);
			} else if (line.startsWith("PRIMER_RIGHT_SEQUENCE")) {
				rightPrimer = line.substring(line.indexOf('=')+1);
			} else if (line.startsWith("PRIMER_LEFT_TM")) {
				leftTm = line.substring(line.indexOf('=')+1);
			} else if (line.startsWith("PRIMER_RIGHT_TM")) {
				rightTm = line.substring(line.indexOf('=')+1);
			} else if (line.startsWith("PRIMER_PRODUCT_SIZE=")) {
				productSize = line.substring(line.indexOf('=')+1);
			} else if (printFlags && line.startsWith("SEQUENCE=")) {
				sequence = line.substring(line.indexOf('=')+1);
				seqInfo = new StringBuffer();
				seqInfo.append(separator);
				seqInfo.append(sequence.substring(RepeatFlagsCalculator.OFFSET-12,sequence.length()-RepeatFlagsCalculator.OFFSET+12));
				seqInfo.append(separator);
				seqInfo.append(RepeatFlagsCalculator.getRepeatFlag(sequence.substring(RepeatFlagsCalculator.OFFSET,sequence.length()-RepeatFlagsCalculator.OFFSET)));
				seqInfo.append(separator);
				seqInfo.append(RepeatFlagsCalculator.getRepeatFlag(sequence));
			}
			
			if(leftPrimer!=null && rightPrimer!=null && leftTm!=null && rightTm!=null && productSize!=null) {
				out.append(seqId+"|left");
				out.append(separator);
				out.append(leftPrimer);
				out.append(separator);
				out.append(leftTm);
				out.append(separator);
				out.append(seqId+"|right");
				out.append(separator);
				out.append(rightPrimer);
				out.append(separator);
				out.append(rightTm);
				out.append(separator);
				out.append(productSize);
				out.append(seqInfo);
				return line;
			}
			line=in.readLine();
		}
		out.append(emptyLine+seqInfo);
		return line;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		Primer3Helper helper = new Primer3Helper();
		if(COMMAND_CREATE_INPUT.equalsIgnoreCase(args[0])) {
			helper.generatePrimer3Input(args[1],args[2],System.out);
		} else if (COMMAND_MERGE_OUTPUT.equalsIgnoreCase(args[0])) {
			helper.mergeOutput(args[1],args[2],System.out);
		}
	}



	

	
}
