package edu.uconn.bioinformatics.epitopeFinder;

import java.io.PrintStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import edu.uconn.ngstools.transcriptome.Transcript;
import edu.uconn.ngstools.variants.CalledSNV;
import edu.uconn.ngstools.variants.filehandlers.PhredScoreHelper;
import java.util.Map;

public class ReportRecord {
	private DecimalFormat decimalFormat = new DecimalFormat("#,##0.00");
	private CalledSNV snv;
	private Transcript transcript;
	private String referenceProtein="";
	private String mutatedProtein="";
	private int transcriptPosition;
	private List<CalledSNV> nearbySnvs;
	private List <EpitopeHit> hits = new ArrayList<EpitopeHit>();
	private static final String SEPARATOR = "\t";
	public ReportRecord (CalledSNV snv) {
		this.snv = snv;
	}
	public ReportRecord (CalledSNV snv, Transcript transcript) {
		this.snv = snv;
		this.setTranscript(transcript);
	}
	public Transcript getTranscript() {
		return transcript;
	}
	public void setTranscript(Transcript transcript) {
		this.transcript = transcript;
	}
	
	public List<EpitopeHit> getHits() {
		return hits;
	}
	public void addHit(EpitopeHit hit) {
		this.hits.add(hit);
	}
	
	public String getReferenceProtein() {
		return referenceProtein;
	}
	public void setReferenceProtein(String referenceProtein) {
		if(referenceProtein == null) {
			this.referenceProtein = "";
		} else {
			this.referenceProtein = referenceProtein;
		}
	}
	public String getMutatedProtein() {
		return mutatedProtein;
	}
	public void setMutatedProtein(String mutatedProtein) {
		if(mutatedProtein==null) {
			this.mutatedProtein = "";
		} else {
			this.mutatedProtein = mutatedProtein;
		}
	}
	
	public int getTranscriptPosition() {
		return transcriptPosition;
	}
	public void setTranscriptPosition(int transcriptPosition) {
		this.transcriptPosition = transcriptPosition;
	}
	
	public List<CalledSNV> getNearbySnvs() {
		return nearbySnvs;
	}
	public void setNearbySnvs(List<CalledSNV> nearbySnvs) {
		this.nearbySnvs = nearbySnvs;
	}
        public Map<String,List<String>> printRecord(PrintStream out,Map<String,List<String>> printedEpitopes, int minScore) {
            for(EpitopeHit hit: hits) {
//cancel filtering by min score
//                if (hit.getScore() < minScore)
//                   continue;
                String mutEpitop = getStringToPrint(hit.getReferenceEpitope(),hit.getEpitope());
//with score filter canceling, we get a lot of records with empty mutant epitope field; skip those
                if (mutEpitop == null || mutEpitop.equals(""))
                    continue;
                List<String> printedEpitopeTranscriptIDs = printedEpitopes.get(mutEpitop);
                String searchString = transcript.getId()+hit.getMotif();
                //System.out.println("in printRecord input"+mutEpitop+" "+transcript.getId());
                // new epitope
//                if (printedEpitopeTranscriptIDs == null || !printedEpitopeTranscriptIDs.contains(transcript.getId())) {
                if (printedEpitopeTranscriptIDs == null || !printedEpitopeTranscriptIDs.contains(searchString)) {
                    if (printedEpitopeTranscriptIDs == null) {
                        printedEpitopeTranscriptIDs = new ArrayList<String>();
//                        printedEpitopeTranscriptIDs.add(transcript.getId());
                        printedEpitopeTranscriptIDs.add(searchString);
                        printedEpitopes.put(mutEpitop, printedEpitopeTranscriptIDs);
                    }
                    else
//                        printedEpitopes.get(mutEpitop).add(transcript.getId());
                        printedEpitopes.get(mutEpitop).add(searchString);
//                    if (hit.getEpitope().startsWith("AYCETCWE"))
//                        System.out.println("adding "+mutEpitop+" "+searchString);
                    printHit(hit,out);
                }
//as per Fei's request. For now, do not check for repetition; always print
//back to uniq                     printHit(hit,out);
//                else
//                    if (hit.getEpitope().startsWith("AYCETCWE"))
//                        System.out.println("found "+mutEpitop+" "+searchString);
            }
            return printedEpitopes;
        }

        public void printHit(EpitopeHit hit, PrintStream out) {
		String snpLine = getSNPLine();
		String statsLine = getStatsLine();
		out.print(snpLine+SEPARATOR);
		out.print(hit.getMotif()+SEPARATOR);
		out.print(hit.getReferenceEpitope()+SEPARATOR);
		out.print(decimalFormat.format(hit.getReferenceScore())+SEPARATOR);
		out.print(getStringToPrint(hit.getReferenceEpitope(), hit.getEpitope())+SEPARATOR);
		out.print(decimalFormat.format(hit.getScore())+SEPARATOR);
		double diff = hit.getScore()-hit.getReferenceScore();
		out.print(decimalFormat.format(diff)+SEPARATOR);
		out.println(statsLine);
       }

	public void printRecord(PrintStream out) {
		String snpLine = getSNPLine();
		String statsLine = getStatsLine();
		for(EpitopeHit hit: hits) {
			out.print(snpLine+SEPARATOR);
			out.print(hit.getMotif()+SEPARATOR);
			out.print(hit.getReferenceEpitope()+SEPARATOR);
			out.print(decimalFormat.format(hit.getReferenceScore())+SEPARATOR);
			out.print(getStringToPrint(hit.getReferenceEpitope(), hit.getEpitope())+SEPARATOR);
			out.print(decimalFormat.format(hit.getScore())+SEPARATOR);
			double diff = hit.getScore()-hit.getReferenceScore();
			out.print(decimalFormat.format(diff)+SEPARATOR);
			out.println(statsLine);
		}
	}
	private String getStatsLine() {
		StringBuffer answer = new StringBuffer();
		double maxDiff=0;
		double minDiff=0;
		double maxScore=Integer.MIN_VALUE;
		for(EpitopeHit hit: hits) {
			double diff = hit.getScore()-hit.getReferenceScore();
			if(maxScore == Integer.MIN_VALUE || diff > maxDiff) {
				maxDiff = diff;
			}
			if(maxScore == Integer.MIN_VALUE || diff < minDiff) {
				minDiff = diff;
			}
			if(maxScore < hit.getScore()) {
				maxScore = hit.getScore();
			}
		}
		answer.append(hits.size());
		answer.append(SEPARATOR);
		answer.append(decimalFormat.format(minDiff));
		answer.append(SEPARATOR);
		answer.append(decimalFormat.format(maxDiff));
		answer.append(SEPARATOR);
		answer.append(decimalFormat.format(maxScore));
		return answer.toString();
	}
	private String getSNPLine () {
		StringBuffer answer = new StringBuffer();
		answer.append(snv.getSequenceName());
		answer.append(SEPARATOR);
		answer.append(snv.getPosition());
		answer.append(SEPARATOR);
		if(transcript!=null) {
			answer.append(transcript.getGene().getName());
			answer.append(SEPARATOR);
			answer.append(transcript.getId());
			answer.append(SEPARATOR);
			answer.append(transcriptPosition);
		} else {
			answer.append("-");
			answer.append(SEPARATOR);
			answer.append("-");
			answer.append(SEPARATOR);
			answer.append("-");
		}
		answer.append(SEPARATOR);
		//UCSC link
		answer.append("http://genome.ucsc.edu/cgi-bin/hgTracks?&db=mm9&position="+snv.getSequenceName()+"%3A"+snv.getPosition());
		answer.append(SEPARATOR);
		if(referenceProtein.length()>0 ) {
			answer.append(referenceProtein);
		} else {
			answer.append("-");
		}
		answer.append(SEPARATOR);
		if(mutatedProtein.length()>0) {
			answer.append(getStringToPrint(referenceProtein, mutatedProtein));
		} else {
			answer.append("-");
		}
		answer.append(SEPARATOR);
		answer.append(snv.getRefBase());
		answer.append(SEPARATOR);
		answer.append(snv.getAltBase());
		
		answer.append(SEPARATOR);
		answer.append(Math.round(snv.getCountReference()));
		answer.append(SEPARATOR);
		answer.append(Math.round(snv.getCountAlternative()));
		answer.append(SEPARATOR);
		answer.append(PhredScoreHelper.calculatePhredScore(1-snv.getProbability()));
		answer.append(SEPARATOR);
		answer.append(snv.getGenotype());
		
		answer.append(SEPARATOR);
		answer.append(getSNVIds(nearbySnvs));
		return answer.toString();
	}
	private String getSNVIds (List<CalledSNV> snvsToPrint) {
		StringBuffer answer = new StringBuffer();
		if(snvsToPrint!=null && snvsToPrint.size()>0) {
			for(int i=0;i<snvsToPrint.size();i++) {
				CalledSNV snv = snvsToPrint.get(i);
				if(i>0) {
					answer.append("-");
				}
				answer.append(snv.getPosition());
			}
		} else {
			answer.append("-");
		}
		return answer.toString();
	}
	private String getStringToPrint(String reference, String modified) {
		char [] answer = new char[modified.length()];
		for(int i=0;i<answer.length;i++) {
			if (i>=reference.length() || modified.charAt(i)!=reference.charAt(i)) {
				answer[i]=Character.toLowerCase(modified.charAt(i));
			} else {
				answer[i]=Character.toUpperCase(modified.charAt(i));
			}
		}
		return new String(answer);
	}
}
