package edu.uconn.bioinformatics.epitopeFinder;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ResultsFilter {
	private String [] alleles;
	String separator = "\t";
	//TODO: Update with new format
	private void filterResults(String reportFilename, PrintStream out) throws IOException {
		FileInputStream fis = new FileInputStream(reportFilename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split(separator);
			List<EpitopeHit> refFilteredPredictions = getFilteredPredictions(items[4],items[5]);
			List<EpitopeHit> modFilteredPredictions = getFilteredPredictions(items[6],items[7]);
			double maxScoreRef = getMaximumScore(refFilteredPredictions);
			double maxScoreMod = getMaximumScore(modFilteredPredictions);
			if(maxScoreMod > 0 && maxScoreRef <= maxScoreMod && !"0".equals(items[10])) {
				for(int j=0;j<5;j++) {
					out.print(items[j]+separator);
				}
				out.print(separator+maxScoreRef+separator+items[6]+separator);
				out.print(separator+maxScoreMod);
				for(int j=8;j<items.length;j++) {
					out.print(separator+items[j]);
				}
				out.println();
				out.flush();
			}
			
			line=in.readLine();
		}
		out.flush();
		in.close();
		
	}
	private double getMaximumScore(List<EpitopeHit> hitsList) {
		double max = 0;
		for(EpitopeHit hit:hitsList) {
			if(max < hit.getScore()) {
				max = hit.getScore();
			}
		}
		return max;
	}
	private List<EpitopeHit> getFilteredPredictions(String protein, String predictionsStr) {
		List<EpitopeHit> answer = new ArrayList<EpitopeHit>();
		if(predictionsStr == null || predictionsStr.length()==0) {
			return answer;
		}
		String [] predArray = predictionsStr.split(","); 
		if(predArray.length==1 && "-".equals(predArray[0])) {
			return answer;
		}
		for(int i=0;i<predArray.length;i++) {
			String [] predValues = predArray[i].split(":");
			if(Arrays.binarySearch(alleles, predValues[1])>=0) {
				answer.add(new EpitopeHit(predValues[1],predValues[0],protein.indexOf(predValues[0]),Integer.parseInt(predValues[2])));
			}
		}
		return answer;
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		ResultsFilter filter = new ResultsFilter();
		filter.alleles = args[1].split(",");
		Arrays.sort(filter.alleles);
		filter.filterResults(args[0],System.out);
	}

}
