package edu.uconn.bioinformatics.epitopeFinder;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

public class SyfpeithiProteinEpitopeChecker implements ProteinEpitopeChecker {
	public static final int NUMBER_OF_TRIES = 10;
	public static final int MAX_WAIT_TIME = 3600000;
	
	@Override
        public void setAlgorithm(String algorithm){
        }

        @Override
	public List<EpitopeHit> findEpitopes(String proteinSequence, double minScore) throws IOException {
		IOException ex = null;
		for (int i=0;i<=NUMBER_OF_TRIES;i++) {
			try {
				//Wait before the next try
				if (i> 0) {
					long waitTime = (i)*MAX_WAIT_TIME / NUMBER_OF_TRIES;
					System.out.println(ex.getMessage()+". Waiting: "+(waitTime/1000)+" seconds to reconnect");
					try {
						Thread.sleep(waitTime);
					} catch (InterruptedException e1) {
						e1.printStackTrace();
						throw ex;
					}
					System.out.println("Trying to reconnect");
				}
				//Try to connect
				URL url = new URL("http://www.syfpeithi.de/scripts/MHCServer.dll/EpitopePrediction?Motif=ALL&amers=0&SEQU="+proteinSequence+"&DoIT=++Run++");
				HttpURLConnection conn = (HttpURLConnection)url.openConnection();
				conn.setRequestMethod("GET");
				InputStream is = (InputStream)conn.getInputStream();
				List<EpitopeHit> epitopes = processHtmlResponse(proteinSequence, is, minScore);
				is.close();
				conn.disconnect();
				return epitopes;
			} catch (IOException e) {
				ex = e;
			}
		}
		throw ex;
	}
	@Override
	public List<EpitopeHit> findEpitopes(String proteinSequence) throws IOException {
		return findEpitopes(proteinSequence,0);
	}
	private List<EpitopeHit> processHtmlResponse(String proteinSequence, InputStream is, double minScore) throws IOException {
		ArrayList<EpitopeHit> hits = new ArrayList<EpitopeHit>();
		BufferedReader in = new BufferedReader(new InputStreamReader(is));
		String line = in.readLine();
		String nextMotif="";
		int nextLength = 0;
		int nextEpitopePos=0;
		int nextScore = 0;
		while(line!=null) {
			if (line.contains("<th ")) {
				int pos = line.indexOf("<a name=");
				if(pos>=0) {
					String motifLine = line.substring(pos);
					pos = motifLine.indexOf(">")+1;
					motifLine = motifLine.substring(pos);
					pos = motifLine.indexOf(' ');
					if(pos>=0) {
						nextMotif = motifLine.substring(0,pos);
						nextLength = getEpitopeLength(motifLine);
					}
				}
			} else if (line.startsWith("<TR ")) {
				int pos = line.indexOf("<td align=right>");
				for(int i=0;pos!=-1;i++) {
					String scoreLine = line.substring(pos);
					int pos2 = scoreLine.indexOf(">")+1;
					scoreLine = scoreLine.substring(pos2);
					pos2 = scoreLine.indexOf("<");
					if(pos2>=0) {
						if(i%2==0 ) {
							nextEpitopePos = Integer.parseInt(scoreLine.substring(0,pos2))-1;
						} else {
							nextScore = Integer.parseInt(scoreLine.substring(0,pos2));
							if(nextScore>=minScore) {
								hits.add(new EpitopeHit(nextMotif, proteinSequence.substring(nextEpitopePos,nextEpitopePos+nextLength), nextEpitopePos, nextScore));
								System.out.println("Motif: "+nextMotif+". Position: "+nextEpitopePos+". Length: "+nextLength+". Score: "+nextScore);
							}
						}
					}
					
					line = line.substring(pos+1);
					pos = line.indexOf("<td align=right>");
				}
			}
			line = in.readLine();
		}
		return hits;
	}
	private int getEpitopeLength(String motifLine) {
		if(motifLine.contains("octamers")) {
			return 8;
		}
		if(motifLine.contains("nonamers")) {
			return 9;
		}
		if(motifLine.contains("decamers")) {
			return 10;
		}
		if(motifLine.contains("11 - mers")) {
			return 11;
		}
		
		return EpitopeHit.MAX_EPITOPE_LENGTH;
	}
	@Override
	public double getScore(String epitope, String motif) throws Exception {
		// TODO Auto-generated method stub
		return 0;
	}
	@Override
	public void setMotifs(String motifs) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public Map<String, List<EpitopeHit>> findEpitopes(Set<String> proteinSequences, double minScore) throws Exception {
		Map<String, List<EpitopeHit>> answer = new TreeMap<String, List<EpitopeHit>>();
		for(String protein: proteinSequences) {
			answer.put(protein, findEpitopes(protein, minScore));
		}
		return answer;
	}
	
	

}
