package edu.uconn.bioinformatics.fileHandlers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import edu.uconn.bioinformatics.haplotyper.Fragment;
import edu.uconn.bioinformatics.mapping.PhasedBlock;
import edu.uconn.bioinformatics.mapping.PhasedGenome;
import edu.uconn.bioinformatics.mapping.PhasedSequence;
import edu.uconn.ngstools.variants.CalledSNV;

public class HaplotypesHandler {
	
	public PhasedGenome loadHaplotypes (List<CalledSNV> snvs, String filename) throws IOException {
		PhasedGenome phasedGenome = new PhasedGenome(snvs);
		FileInputStream fis = new FileInputStream(filename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		int lastNumBlock =0;
		int posSNP =0;
		List<CalledSNV> blockSNVs = new ArrayList<CalledSNV>();
		StringBuilder haplotype = new StringBuilder();
		PhasedSequence pSeq = null;
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split("\t");
			String seqName = items[0];
			int absPos = Integer.parseInt(items[1]);
			char a1 = items[2].charAt(0);
			int numBlock = Integer.parseInt(items[4]);
			if(numBlock!=lastNumBlock && pSeq!=null && haplotype.length()>1) {
				pSeq.addBlock(blockSNVs, haplotype.toString());
				blockSNVs = new ArrayList<CalledSNV>();
				haplotype = new StringBuilder();
			}
			if(pSeq==null || !seqName.equalsIgnoreCase(pSeq.getSequenceName())) {
				pSeq = phasedGenome.getPhasedSequence(seqName);
				posSNP =0;
			}
			if(pSeq!=null) {
				if(a1!=Fragment.NODATACHAR) {
					int pos = pSeq.findSNVPos(absPos,posSNP);
					if(pos>=0) {
						CalledSNV snp = pSeq.getSnvs().get(pos);
						blockSNVs.add(snp);
						haplotype.append(a1);
					}
					lastNumBlock = numBlock;
				}
			}
			line=in.readLine();
		}
		fis.close();
		if(pSeq!=null && haplotype.length()>1) {
			pSeq.addBlock(blockSNVs, haplotype.toString());
		}
		return phasedGenome;
		
	}
	public void printHaplotypes(PhasedGenome phasedGenome, PrintStream out) {
		int numBlock=1;
		for(PhasedSequence phasedSeq:phasedGenome.getSequenceHaplotypes()) {
			
			for(PhasedBlock b:phasedSeq.getHaplotypeBlocks()) {
				List<CalledSNV> blockSNVs = b.getSnvs();
				String haplotype = b.getHaplotype();
				for(int i=0;i<blockSNVs.size();i++) {
					CalledSNV snp = blockSNVs.get(i);
					char a1 = haplotype.charAt(i);
					char a2 = Fragment.NODATACHAR;
					if(a1 == Fragment.ALLELE1CHAR) {
						a2 = Fragment.ALLELE2CHAR;
					} else if (a1 == Fragment.ALLELE2CHAR) {
						a2 = Fragment.ALLELE1CHAR;
					}
					out.println(snp.getSequenceName()+"\t"+snp.getPosition()+"\t"+a1+"\t"+a2+"\t"+numBlock);
				}
				numBlock++;
			}
		}
		out.flush();
	}
}
