package edu.uconn.bioinformatics.fileHandlers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import edu.uconn.ngstools.sequences.DNASequence;
import edu.uconn.ngstools.variants.SNV;

public class MPDSNPSHandler {
	private String separator = "\t";
	private String referenceStrain;
	public MPDSNPSHandler(String referenceStrain) {
		super();
		this.setReferenceStrain(referenceStrain);
	}
	public List<SNV> loadSNVS(String snpsFilename) throws IOException {
		List<SNV> answer = new ArrayList<SNV>();
		FileInputStream fis = new FileInputStream(snpsFilename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		String line=in.readLine();
		int referenceColumn = getReferenceColumn(line);
		if(referenceColumn<0) return answer;
		line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split(separator);
			String chr = items[0];
			try {
				chr = String.valueOf(Integer.parseInt(chr));
			} catch (NumberFormatException e) {
			}
			int position = (int) (1000000.0*Double.parseDouble(items[1]));
			char altBase = items[referenceColumn].charAt(0);
			if(DNASequence.isInAlphabeth(altBase) && items[3].length()==3) {
				char refBase = items[3].charAt(0);
				if(altBase == refBase) {
					refBase = items[3].charAt(2);
				}
				SNV snp = new SNV(chr,position,refBase,altBase);
				answer.add(snp);
			}
			line=in.readLine();
		}
		fis.close();
		return answer;
	}
	private int getReferenceColumn (String line) {
		String [] items = line.split(separator);
		for(int i=0;i<items.length;i++) {
			if(items[i].equals(referenceStrain)) {
				return i;
			}
		}
		return -1;
	}
	public String getReferenceStrain() {
		return referenceStrain;
	}
	public void setReferenceStrain(String referenceStrain) {
		this.referenceStrain = referenceStrain;
	}

}
