package edu.uconn.bioinformatics.fileHandlers;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import edu.uconn.ngstools.readalignments.AlignmentsTranslator;
import edu.uconn.ngstools.readalignments.Read;
import edu.uconn.ngstools.readalignments.ReadAlignment;


public class MaqReadAlignmentsHandler  {
	private AlignmentsTranslator alignmentsTranslator;
	//List of sequence names in the same order as they appear in the contigs file
	private List<String> sequenceNames = new ArrayList<String>();
	private short minQuality = 0;
	
	public List<ReadAlignment> loadReadAlignments(String filename) throws IOException {
		List<ReadAlignment> alns = new ArrayList<ReadAlignment>();
		FileInputStream fis = new FileInputStream(filename);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			ReadAlignment aln = createAlignment(line);
			if(aln!=null)alns.add(aln);
			line=in.readLine();
		}
		fis.close();
		return alns;
	}
	private String lastSeqName=null;
	private ReadAlignment createAlignment(String line) {
		String [] items = line.split(" |\t");
		String id = items[0];
		Read r = new Read(id,items[14].toUpperCase(),items[15]);
		short alnQuality = Short.parseShort(items[6]);
		if ("130".equals(items[5])) {
			alnQuality = Short.parseShort(items[8]);
		}
		//TODO: Handle paired end reads
		if(!items[1].equals(lastSeqName)) {
			lastSeqName = items[1];
			sequenceNames.add(lastSeqName);
		}
		int relativeStart = Integer.parseInt(items[2])-1;
		String cigar = buildCigar(r, items);
		ReadAlignment aln = new ReadAlignment(r,lastSeqName,relativeStart,cigar,alnQuality);
		if(alignmentsTranslator!=null) alignmentsTranslator.translateAlignment(aln);
		return aln;
	}
	

	private String buildCigar(Read r, String[] items) {
		if("130".equals(items[5])&& !"0".equals(items[6])&& !"0".equals(items[7])) {
			int eventStart = Integer.parseInt(items[6]);
			int eventLength = Integer.parseInt(items[7]);
			StringBuffer cigar = new StringBuffer();
			cigar.append(""+eventStart+"M");
			int remaining = r.length()-eventStart;
			if(eventLength>0) {
				cigar.append(""+eventLength+"I");
				remaining-=eventLength;
			} else {
				cigar.append(""+(-eventLength)+"D");
			}
			if(remaining>0) {
				cigar.append(""+remaining+"M");
			}
			return cigar.toString();
		}
		return ""+r.length()+"M";
	}

	

	public AlignmentsTranslator getAlignmentsTranslator() {
		return alignmentsTranslator;
	}


	public void setAlignmentsTranslator(AlignmentsTranslator alignmentsTranslator) {
		this.alignmentsTranslator = alignmentsTranslator;
	}


	public short getMinQuality() {
		return minQuality;
	}

	public void setMinQuality(short minQuality) {
		this.minQuality = minQuality;
	}
	/**
	 * @return the sequenceNames
	 */
	public List<String> getSequenceNames() {
		return sequenceNames;
	}


	public void setSequenceNames(List<String> sequenceNames) {
		this.sequenceNames = sequenceNames;
	}
	
}