package edu.uconn.bioinformatics.mapping;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.uconn.bioinformatics.haplotyper.Fragment;
import edu.uconn.ngstools.genome.GenomicRegion;
import edu.uconn.ngstools.genome.GenomicRegionPositionComparator;
import edu.uconn.ngstools.variants.CalledSNV;


public class PhasedBlock implements GenomicRegion{
	private List<CalledSNV> snvs;
	private String haplotype;
	private int start;
	private int end;
	public PhasedBlock(List<CalledSNV> snvs, String haplotype) {
		super();
		assert snvs.size() == haplotype.length();
		this.snvs = snvs;
		this.haplotype = haplotype;
		start = snvs.get(0).getPosition();
		end = snvs.get(snvs.size()-1).getPosition(); 
	}

	public List<CalledSNV> getSnvs() {
		return snvs;
	}
	public String getHaplotype() {
		return haplotype;
	}
	@Override
	public int getEnd() {
		return end;
	}
	@Override
	public String getSequenceName() {
		return snvs.get(0).getSequenceName();
	}
	@Override
	public int getStart() {
		return start;
	}
	@Override
	public int length() {
		return end - start +1;
	}
	public List<CalledSNV> findSNVsInPhase(CalledSNV searchSnv) {
		List<CalledSNV> snvsInPhase = new ArrayList<CalledSNV>();
		int index = Collections.binarySearch(snvs, searchSnv, GenomicRegionPositionComparator.getInstance());
		if(index<0) {
			return snvsInPhase;
		}
		char allele = haplotype.charAt(index);
		if(allele== Fragment.NODATACHAR) {
			return snvsInPhase;
		}
		
		for(int i=0;i<snvs.size();i++) {
			char a2 = haplotype.charAt(i);
			if(allele == a2) {
				snvsInPhase.add(snvs.get(i));
			}
		}
		return snvsInPhase;
	}

	public boolean containsSNV(CalledSNV searchSnv) {
		int index = Collections.binarySearch(snvs, searchSnv, GenomicRegionPositionComparator.getInstance());
		if(index<0) {
			return false;
		}
		char allele = haplotype.charAt(index);
		if(allele== Fragment.NODATACHAR) {
			return false;
		}
		return true;
	}
}
