package edu.uconn.bioinformatics.mapping;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import edu.uconn.ngstools.variants.CalledSNV;


public class PhasedGenome {
	private List<PhasedSequence> sequenceHaplotypes;
	private Map<String, PhasedSequence> sequenceHaplotypesMap = new TreeMap<String, PhasedSequence>();
	
	public PhasedGenome (List<CalledSNV> snvs) {
		setSNVS(snvs);
	}
	
	public List<PhasedSequence> getSequenceHaplotypes() {
		return sequenceHaplotypes;
	}

	public int getNumSequences() {
		return sequenceHaplotypes.size();
	}
	private PhasedSequence lastPhasedSequenceQueried = null;
	public PhasedSequence getPhasedSequence(String seqName) {
		if(lastPhasedSequenceQueried==null || !lastPhasedSequenceQueried.getSequenceName().equals(seqName)) {
			lastPhasedSequenceQueried = sequenceHaplotypesMap.get(seqName); 
		}
		return lastPhasedSequenceQueried;
	}
	public void setSNVS(List<CalledSNV> snvs) {
		sequenceHaplotypes = new ArrayList<PhasedSequence>();
		String seqName = null;
		List<CalledSNV> seqSNVS = null;
		for(CalledSNV snv:snvs) {
			if(!snv.getSequenceName().equalsIgnoreCase(seqName)) {
				if(seqName!=null && seqSNVS!=null && seqSNVS.size()>0) {
					PhasedSequence pSeq = new PhasedSequence(seqName,seqSNVS);
					sequenceHaplotypes.add(pSeq);
					sequenceHaplotypesMap.put(seqName, pSeq);
				}
				seqName = snv.getSequenceName();
				seqSNVS = new ArrayList<CalledSNV>();
			}
			if(snv.isHeterozygous()) {
				seqSNVS.add(snv);
			}
		}
		if(seqName!=null && seqSNVS!=null && seqSNVS.size()>0) {
			PhasedSequence pSeq = new PhasedSequence(seqName,seqSNVS);
			sequenceHaplotypes.add(pSeq);
			sequenceHaplotypesMap.put(seqName, pSeq);
		}
	}
}
