package edu.uconn.bioinformatics.mapping;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

import edu.uconn.ngstools.genome.GenomeAssembly;
import edu.uconn.ngstools.sequences.Sequence;

/**
 * @author Jorge Duitama
 */
public class RepeatFlagsCalculator {
	public static final int OFFSET = 400;
	static String separator = "\t";
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		GenomeAssembly genomeAssembly = new GenomeAssembly(args[0]);
		FileInputStream fis = new FileInputStream(args[1]);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		PrintStream out = System.out;
		String line=in.readLine();
		for(int i=0;line!=null;i++) {
			String [] items = line.split(separator);
			String seq = items[0];
			int pos = Integer.parseInt(items[1]);
			String sequence = getSequenceForPrimersDesign(genomeAssembly, seq, pos, OFFSET);
			out.print(line);
			out.print(separator);
			out.print(getRepeatFlag(sequence.substring(RepeatFlagsCalculator.OFFSET,sequence.length()-RepeatFlagsCalculator.OFFSET)));
			out.print(separator);
			out.print(getRepeatFlag(sequence));
			out.println();
			line=in.readLine();
		}
		out.flush();
		in.close();
	}
	/**
	 * Generates the sequence surrounding an interval which can be used for primers design flanking the region of
	 * interest
	 * @param genome Genome to look for
	 * @param seqName Name of the sequence to look for
	 * @param productPos position that is meant to be included in the PCR product
	 * @param offset Extra bases on each side that will be used for primers design
	 * @return String Sequence including the bases for primers design surrounding the desired product
	 */
	public static String getSequenceForPrimersDesign(GenomeAssembly genome, String seqName, int productPos, int offset) {
		Sequence sequence = genome.getSequence(seqName);
		if(productPos < 0 || productPos>=  sequence.length()) {
			return "-";
		}
		StringBuffer answer = new StringBuffer();
		int first = productPos-offset-1;
		while (first < 0) {
			answer.append('N');
			first++;
		}
		int end = productPos + offset;
		int endInSequence = Math.min(sequence.length(), end);
		answer.append(sequence.substring(first, endInSequence));
		while (endInSequence < end) {
			answer.append('N');
			endInSequence++;
		}
		return answer.toString();
	}
	public static char getRepeatFlag(String sequence) {
		String upSeq = sequence.toUpperCase();
		if(!upSeq.equals(sequence)) {
			return 'Y';
		}
		return 'N';
	}

}
