package edu.uconn.bioinformatics.mapping;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import edu.uconn.bioinformatics.fileHandlers.HaplotypesHandler;
import edu.uconn.bioinformatics.haplotyper.Block;
import edu.uconn.bioinformatics.haplotyper.Fragment;
import edu.uconn.bioinformatics.haplotyper.FragmentsComparator;
import edu.uconn.bioinformatics.haplotyper.Haplotyper;
import edu.uconn.ngstools.readalignments.ReadAlignment;
import edu.uconn.ngstools.readalignments.filehandlers.SAMReadAlignmentsHandler;
import edu.uconn.ngstools.variants.CalledSNV;
import edu.uconn.ngstools.variants.filehandlers.CalledSNVSFileHandler;

public class SNPSHaplotyper {
	private List<CalledSNV> snvs;

	public List<CalledSNV> getSnvs() {
		return snvs;
	}

	public void setSnvs(List<CalledSNV> snvs) {
		this.snvs = new ArrayList<CalledSNV>();
		for(CalledSNV snv:snvs) {
			if(snv.isHeterozygous()) {
				this.snvs.add(snv);
			}
		}
	}
	public PhasedGenome phaseSNPs(String alignmentsFile) throws Exception { //IOException {
		SAMReadAlignmentsHandler samHandler = new SAMReadAlignmentsHandler();
		Haplotyper haplotyper = new Haplotyper();
		PhasedGenome phasedGenome = new PhasedGenome(snvs);
		FileInputStream fis = new FileInputStream(alignmentsFile);
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		List<Fragment> sequenceFragments=new ArrayList<Fragment>();
		PhasedSequence pSeq = null;
		int lastStart=0;
		ReadAlignment aln = samHandler.createReadAlignment(in);
		String seqIgnore = "";
		for(int i=0;aln!=null;i++) {
			boolean processAln = !aln.getSequenceName().equalsIgnoreCase(seqIgnore);
			if(processAln && (pSeq == null || !aln.getSequenceName().equalsIgnoreCase(pSeq.getSequenceName()))) {
				if(pSeq != null && sequenceFragments.size()>0) {
					phaseFragments(haplotyper, sequenceFragments, pSeq);
					sequenceFragments=new ArrayList<Fragment>();
					lastStart = 0;
				}
				
				pSeq = phasedGenome.getPhasedSequence(aln.getSequenceName()); 
				if(pSeq == null) {
					seqIgnore = aln.getSequenceName();
					processAln = false;
				} else {
					seqIgnore = "";
				}
			}
			if(processAln) {
				Fragment f = pSeq.buildFragment(aln);
				if(f!=null) {
					if(aln.getStart()!=lastStart) {
						sequenceFragments.add(f);
						lastStart = aln.getStart();
					}
				}
			}
			if(i>0 && i%100000==0) System.out.println("Processed "+i+" alignments");
			aln = samHandler.createReadAlignment(in);
		}
		if(pSeq!=null && sequenceFragments.size()>0 ) {
			phaseFragments(haplotyper, sequenceFragments, pSeq);
		}
		return phasedGenome;
	}

	private void phaseFragments(Haplotyper haplotyper, List<Fragment> sequenceFragments, PhasedSequence pSeq) throws Exception{
		Collections.sort(sequenceFragments,new FragmentsComparator());
		System.out.println("Calculating haplotypes for sequence: "+pSeq.getSequenceName()+". Fragments: "+sequenceFragments.size());
		List<Block> seqBlocks = haplotyper.buildHaplotypes(sequenceFragments);
		System.out.println("Sequence phased in "+seqBlocks.size()+" blocks");
		if(seqBlocks.size()>0) {
			pSeq.setHaplotypeBlocks(seqBlocks);
		}
	}
	public static void main(String[] args) throws Exception {
		SNPSHaplotyper haplotyper = new SNPSHaplotyper();
		CalledSNVSFileHandler snvsHandler = new CalledSNVSFileHandler();
		HaplotypesHandler hapsHandler = new HaplotypesHandler();
		List<CalledSNV> snvs = snvsHandler.loadCalledSNVS(args[0]);
		haplotyper.setSnvs(snvs);
		PhasedGenome phasedGenome = haplotyper.phaseSNPs(args[1]);
		PrintStream out = new PrintStream(args[2]);
		hapsHandler.printHaplotypes(phasedGenome,out);
		out.close();
	}
}
