package edu.uconn.bioinformatics.programs;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import edu.uconn.ngstools.readalignments.ReadAlignment;
import edu.uconn.ngstools.readalignments.filehandlers.SAMReadAlignmentsHandler;

public class RepeatedReadsFilterConsensus {

	/**
	 * @param args
         *
	 */
        private static boolean phred = false;
        
	public static void main(String[] args) throws Exception {
		SAMReadAlignmentsHandler samHandler = new SAMReadAlignmentsHandler();
		List<ReadAlignment> alns = new ArrayList<ReadAlignment>();
		String lastSeqName = null;
		int lastStart =0;
		int lastEnd = 0;
                short lastFlags = 0;
		String lastCigar = null;
		String lastLine = null;
                ReadAlignment lastAln;
		FileInputStream fis = new FileInputStream(args[0]);
                int n = 1;
                boolean consensus = false;
                while (n < args.length) {
                    if (args[n].equals("-c"))
                        consensus = true;
                    if (args[n].equals("-phred"))
                        phred=true;
                    n++;
                }
  //              System.err.println("consensus = "+consensus);
		PrintStream out = System.out;
		BufferedReader in = new BufferedReader(new InputStreamReader(fis));
		String line = in.readLine();
		while(line!=null) {
			if(line.startsWith("@")) {
				out.println(line);
			}
                        else if (!consensus) {
//                                System.err.println("in !consensus = ");

				ReadAlignment aln = samHandler.createReadAlignment(line);
				if(aln!=null) {
                                       if (phred)
                                            printAln(aln,out,convertBaseQScore(aln));
                                       else
                                           printAln(aln,out,aln.getRead().getQualityScores());
                                }
                        }
                        else {
    //                            System.err.println("in else (consensus = true)");
				ReadAlignment aln = samHandler.createReadAlignment(line);
				if(aln!=null) {
					if(!aln.getSequenceName().equals(lastSeqName) || aln.getStart()!=lastStart || aln.getEnd() !=lastEnd || !aln.getCigar().equals(lastCigar) || (aln.getFlags() != lastFlags)) {
						if(alns.size()==1) {
                                                    //out.println(lastLine);
                                                   if (phred)
                                                       printAln(alns.get(0),out,convertBaseQScore(alns.get(0)));
                                                    else
                                                       printAln(alns.get(0),out,alns.get(0).getRead().getQualityScores());
                                                }
						else  {
      //                                              System.err.println("in else: alns.size = "+alns.size());

                                                    printConsensus(alns,out);
                                                }
						out.flush();
						alns.clear();
					}
					alns.add(aln);
					lastSeqName = aln.getSequenceName();
					lastStart = aln.getStart();
					lastEnd = aln.getEnd();
					lastCigar = aln.getCigar();
                                        lastFlags = aln.getFlags();
 					lastLine = line;
				}
			}
			line = in.readLine();
		}
		in.close();
		if(alns.size()==1) {
                    //out.println(lastLine);
		     ReadAlignment aln = samHandler.createReadAlignment(line);
        	     if(aln!=null) {
                          if (phred)
                             printAln(alns.get(0),out,convertBaseQScore(aln));
                          else
                             printAln(alns.get(0),out,alns.get(0).getRead().getQualityScores());
                    }
                }
		else printConsensus(alns,out);


	}
        private static void printAln(ReadAlignment aln, PrintStream out, String qualityScores) {
		out.print(aln.getRead().getId()+"\t"+aln.getFlags()+"\t"+aln.getSequenceName()+"\t"+aln.getStart()+"\t"+aln.getMappingQuality());
		out.println("\t"+aln.getCigar()+"\t*\t0\t0\t"+aln.getRead().toString()+"\t"+qualityScores);
            
        }
        private static String convertBaseQScore(ReadAlignment aln) {
            StringBuilder phredScores = new StringBuilder();

            String QScores=aln.getRead().getQualityScores();
//            System.err.println(QScores);
            for (int i=0; i<QScores.length(); i++) {
                int Phred = (int) QScores.charAt(i) - 31;
                phredScores.append((char)Phred);
                //System.err.println("i = "+i+" BaseQ = "+QScores.charAt(i)+" Phred64 = "+((int) QScores.charAt(i))+" Phred33 = "+Phred+" new BaseQ = "+(char)Phred);
            }
            //aln.getRead().setQualityScores(phredScores.toString());
            return phredScores.toString();

        }

	private static void printConsensus(List<ReadAlignment> alns, PrintStream out) {
		String bases = "ACGT";
        //        System.err.println("in printConsensus: alns.size = "+alns.size());
		if(alns.size()<2) return;
		ReadAlignment firstAln = alns.get(0);
		int readLength = firstAln.getRead().length();
		int [][] counts = new int [readLength][4];
		for(int i=0;i<alns.size();i++) {
			String read = alns.get(i).getRead().toString().toUpperCase();
			for(int j=0;j<read.length();j++) {
				char c = read.charAt(j);
				int idx = bases.indexOf(c);
				if(idx>=0) counts[j][idx]++;
			}
		}
		StringBuilder consensus = new StringBuilder();
		for (int i=0;i<counts.length;i++) {
			int maxJ = 0;
			boolean shared = false;
			for (int j=1;j<counts[0].length;j++) {
				if(counts[i][j]>counts[i][maxJ]) {
					maxJ = j;
					shared= false;
				} else if(counts[i][j]==counts[i][maxJ]) {
					shared = true;
				}
			}
			if(shared) consensus.append('N');
			else consensus.append(bases.charAt(maxJ));
		}
                String qualityScores;
                if (phred)
                    qualityScores = convertBaseQScore(firstAln);
                else
                    qualityScores = firstAln.getRead().getQualityScores();
		out.print(firstAln.getRead().getId()+"_consensus"+"\t"+firstAln.getFlags()+"\t"+firstAln.getSequenceName()+"\t"+firstAln.getStart()+"\t"+firstAln.getMappingQuality());
		out.println("\t"+firstAln.getCigar()+"\t*\t0\t0\t"+consensus.toString()+"\t"+qualityScores);
	}
}
